
import React from 'react';
import PropTypes from 'prop-types'
// console.log(React.PropTypes)

import Lab from './lab'
import AuthBar from './authbar'

// a singletons, used elsewhere
import GistDataStore from '../lib/data_store_gist'
import LocalCredStore from '../lib/data_store_cred'

import {check_js_files_syntax, check_if_invalid_syntax} from '../lib/util'

export default class GistLab extends React.Component {
  constructor(props){
    super(props)
    let id = this.props.id || this.props.gist_id
    // this.props.gist_id is static, as is this.state.id whereas this.state.gist_id may change, when a non-owner saves or forks
    // or when a page based lab with props.initial_files and no gist_id becomes a gist
    this.state = {id,
      active_edit: this.props.initial_active_edit,
      // will be filled in with files
      files:false,
      // are we posting to the api, indicator, disable save
      is_sending: false,
      // unsaved changes?
      is_dirty: false,
      // have we ever saved?
      has_saved: false,
      gist_meta: { },
      user_data: { },
      api_error: false,
      // 0 is unloaded, false is no user version found
      // if there is one, the gist id,
      // which we'll load
      user_version: 0,
      // only applies when there's a user version
      // either
      // original_meta: false,
    }
    // you might want to use this in dev, and for the authoring
    // if (this.props.disable_local_storage) { LocalCredStore.set_disabled() }
    this.onBeforeChange = this.onBeforeChange.bind(this)
    this.onLoginSuccess = this.onLoginSuccess.bind(this)
    this.switchFile = this.switchFile.bind(this)
    this.addNewFile = this.addNewFile.bind(this)
    this.onLogOut = this.onLogOut.bind(this)
    this.onUserGistLookup = this.onUserGistLookup.bind(this)
    this.submitWorkClicked = this.submitWorkClicked.bind(this)
    GistDataStore.ee.on("login", this.onLoginSuccess )
    GistDataStore.ee.on("logout", this.onLogOut )
    GistDataStore.ee.on("user_gist_lookup_loaded", this.onUserGistLookup )
  }
  componentDidMount(){
    GistDataStore.maybe_login()
    this.loadInitialFileData()
  }
  loadInitialFileData(){
    // set some state and move on, otherwise, load it from github
    if (this.props.initial_files) {
      // add it if we don't have it and the default prop isn't overridden
      let files = this.maybeAddIndexHTML(this.props.initial_files)
      // try the given filename (defaults to index.js)
      // if no file, fallback to index.html
      let active_edit = files[this.props.initial_active_edit] ? this.props.initial_active_edit : "index.html"
      this.setState({active_edit, files})
    } // we don't have the files cached or as props, load from github
    else if (this.props.gist_id) {
      this.load_from_gist_id(this.props.gist_id)
    } else { this.setState({api_error: "Error Loading Files"}) }
  }
  load_from_gist_id(gist_id){
    GistDataStore.get_gist_data(gist_id).then( ({files, meta}) =>{
      files = this.maybeAddIndexHTML(files)
      let active_edit = files[this.props.initial_active_edit] ? this.props.initial_active_edit : "index.html"
      this.setState({files, active_edit, gist_meta:meta})
    }).catch( (e)=>{
      this.setState({api_error: `Error Loading Gist`})
    })
  }
  maybeAddIndexHTML(files){
    if (!files['index.html'] && this.props.index_html_default_content){
      files['index.html'] = {'content': this.props.index_html_default_content}
    }
    return files
  }
  onLoginSuccess(user_data){
    // let api_error = false
    // console.log("siccosoz")

    // this.setState({user_data, api_error})
    this.setState({user_data})
    // this should catch when we're over limit and then we login,
    // we need to try again
    if (this.state.api_error && Object.keys(this.state.files).length ==0 ){
      this.setState({api_error: false})
      this.loadInitialFileData()
    } else {
      // normal successful login
      this.setState({api_error: false})
    }
  }
  loginFormSubmit(login_data){
    // this will also call the event emitter when done, for all of instances
    GistDataStore.login(login_data).then( z=>{
      if (login_data.remember_me) { LocalCredStore.set_cred(login_data)}
      // we don't actually update it with the info, see onLoginSuccess
      this.setState({api_error:false})
    }).catch( err=>{
      this.setState({api_error: "Error Logging in to Github"})
    })
  }
  componentWillUnmount(){
    GistDataStore.ee.removeListener("login", this.onLoginSuccess)
    GistDataStore.ee.removeListener("logout", this.onLogOut)
    GistDataStore.ee.removeListener("user_gist_lookup_loaded", this.onUserGistLookup )

  }
  onBeforeChange(editor, data, value){
    let files = this.state.files
    files[this.state.active_edit]["content"] = value
    let is_dirty = true
    this.setState({files, is_dirty})
  }
  updateExisting(){
    this.setState({is_sending:true})
    // if it's originally the user's gist, use the original description
    // otherwise use the title and url, so we can find it later
    let description = this.state.gist_meta.id != this.props.gist_id ? this.get_description_key() : this.state.gist_meta.description
    // not props.gist_id, use state one as it may have changed if we forked
    GistDataStore.update_own_gist(this.state.gist_meta.id, this.state.files, description)
    .then( (data)=>{
      this.setState( {is_dirty: false, is_sending:false, api_error: false, has_saved:true} )
    }).catch( err=>{
      this.setState({api_error: "Error Saving Gist"})
    })
  }
  saveAsNew(){
    this.setState({is_sending:true})
    // let description = this.state.gist_meta.description || ""
    let description = this.get_description_key()
    GistDataStore.create_new_gist(this.state.files, description).then( (gist_meta)=>{
      let api_error = false
      this.setState({gist_meta, api_error, is_sending:false, is_dirty: false, has_saved:true})
    }).catch( err=>{ this.setState({api_error: "Error Saving as New Gist"}) })
  }
  switchFile(filename){
    let active_edit = filename
    this.setState({active_edit})
  }
  // we trigger onLogOut for all instances this way
  logOutClicked(){
    let result = window.confirm("Are you sure you want to log out? You'll lose anything not saved to github!")
    if (!result) {return}
    LocalCredStore.remove_cred()
    GistDataStore.triggerLogOut()
    // this.setState( {user_data:{}} )
    // localStorage.clear()
  }
  // this gets called even when this lab is not the originator
  onLogOut(){
    // don't need to do this unless they have a version.
    this.setState({user_version:false, gist_meta:{}, user_data:{}, has_saved:false, is_dirty:false})
    this.loadInitialFileData()
  }
  saveClicked(e){
    e.preventDefault()
    if (this.state.gist_meta.owner && this.state.gist_meta.owner == this.state.user_data.username){
      this.updateExisting()
    } else {
      this.saveAsNew()
    }
  }
  // our desc for keeping track of what's based on what
  // don't use `|||` in titles in the inpage versions, or in gist descriptions
  get_description_key(){
    return `${this.props.title}${this.props.description_key_sep}${location.toString()}`
  }
  onUserGistLookup(gist_lookup){
    let ids = gist_lookup[ this.get_description_key() ]
    if (!ids) {
      // this.setState({user_version:false})
      this.setState({user_version:false})
      return
    } if (ids.length == 1) {
      let user_version = ids[0]
      this.setState({user_version})
      this.load_from_gist_id(user_version)
      // let gist_meta = {id:user_version}
      // this.setState({gist_meta})
      // this.setState({user_version})

    } else if (ids.length){
      console.log('TOO MANY MATCHING IDS, need to make them pick')
    }

  }
  revert(e){
    if (e) { e.preventDefault() }
    let result = window.confirm("Are you sure you want to revert to the original in the page? Changes not saved to github will be lost.")
    if (result) {
      this.setState({user_version:false, gist_meta:{}, user_data:{}, has_saved:false, is_dirty:false})
      this.loadInitialFileData()
    }
  }
  submitWorkClicked(){
    console.log('submit work clicked!')
  }
  addNewFile(){
    let name = prompt("Name it:")
    if (!name){return}
    let files = this.state.files
    files[name] = {content:''}
    // console.log(name)
    this.setState({files, active_edit:name})
  }
  render(){
    let addNewFile = this.props.show_new_file_button ? this.addNewFile : null
    let submitWorkClicked = this.props.submit_work_url ? this.submitWorkClicked : null

    let lab_or_loading = !this.state.files ? <div className="loading_lab_box">{this.state.api_error ? ``: `LOADING`}</div> :
      <Lab {...this.state} {...this.props.lab_props} onBeforeChange={this.onBeforeChange} switchFile={this.switchFile} addNewFile={addNewFile}/>

    return <div className="gist_lab">
      <AuthBar
        {...this.state}
        loginFormSubmit={this.loginFormSubmit.bind(this)}
        submitWorkClicked={submitWorkClicked}
        saveClicked={this.saveClicked.bind(this)}
        revert={this.revert.bind(this)}
        logOut={this.logOutClicked.bind(this)}/>
      {!this.state.api_error ? null : <div className="api_error">⚠️ {this.state.api_error}<br/>{this.state.user_data.username ? ``: `You may have used up the anonymous GitHub API quoata. Login to fix this.`}</div> }
      {lab_or_loading}
    </div>
  }
}

const index_html_default_content = `<!DOCTYPE html><html><head></head><body><script type="text/javascript" src="index.js"></script></body></html>`

GistLab.defaultProps = {
  initial_active_edit: 'index.js',
  show_new_file_button: true,
  index_html_default_content,
  description_key_sep: ' ||| ',
  submit_work_url: false, // submitWorkClicked
  // fork_to_save: true,
  // files_shown_white_list: false, // [] for hiding it
  // not impl yet:
  // show_save_as_new_button: true,
  // show_fork_button: true,
  // allow_anonymous: false,
  // babel_preset: '',
}
