
import React from 'react';
import PropTypes from 'prop-types'
import MagicSandbox from '@zischwartz/magic-sandbox'
// let Babel = require("babel-standalone")

import Frame from './frame'
import ConsoleView from './console_view'
import Editor from './editor'

import FileBar from './filebar'
import {check_js_files_syntax, check_if_invalid_syntax} from '../lib/util';

require("../style/lab.less")

// Our mostly dumb Lab Component
// Minimal state, just the console output and whether or not auto_update is
// checked.
//
export default class Lab extends React.Component {
  constructor(props){
    super(props)
    this.state = {
      output_lines:[],
      auto_update:false,
    }
    this.iframe_message_listener = this.iframe_message_listener.bind(this)
    // this.onBeforeChange = this.onBeforeChange.bind(this)
    this.moveCursor = this.moveCursor.bind(this)
  }
  componentDidCatch(error, info) {
    console.log('DID CATCH ERROR')
  }
  componentWillReceiveProps(nextProps){
    // need something like this here for secondary load of user work and logout and revert
    // otherwise it won't update
    if (!this.props.gist_meta && !nextProps.gist_meta) {
      return
    } else if (this.props.gist_meta.id != nextProps.gist_meta.id){
      // this can cause auto update if called at the wrong time
      this.run_code()
    }
  }
  componentDidMount(){
    this.setup_frame_message_listener()
  }
  setup_frame_message_listener(){
      window.addEventListener("message", this.iframe_message_listener, false)
  }
  iframe_message_listener(event){
    if (!event.source) { return }
    if (!event.source.frameElement) { return }
    let frame_id = event.source.frameElement.id
    // unforch this fires for all of them, so we filter by frames contained in this gist, otherwise they'd all update
    let html_ids = Object.keys(this.props.files).filter( (x)=>x.endsWith(".html")).map( filename => `${filename}_${this.props.id}`.replace(/\./g, '') )
    // listener will fire for everything, we only care about these
    if (!html_ids.includes(frame_id)) { return }
    if (!['console-log', 'console-fail', 'runtime-error'].includes(event.data.type) ) { return }
    // if we're editing an html file, assume that's where the error is, and adjust it by the location of the script tag
    let line_number_offset = !this.props.active_edit.endsWith(".html") ? 0 : this.get_line_offset()
    // console.log({line_number_offset})
    let line_number = event.data.lineNumber+line_number_offset
    let line = {line:line_number, type: event.data.type, data: event.data.message, frame_id, column: event.data.colNumber}
    let output_lines = this.state.output_lines
    output_lines.push(line)
    this.setState({output_lines})
  }
  get_line_offset(){
    let content = this.props.files[this.props.active_edit]["content"]
    let pos = content.lastIndexOf("<script>")
    return content.slice(0, pos).split('\n').length-1
  }
  componentWillUnmount(){
    window.removeEventListener("message", this.iframe_message_listener)
  }
  run_code(){
    // clear our output
    this.setState({output_lines:[]})
    // make the frame call setContent with the current props
    // hacky, but better than alternatives
    // this assumes the only refs we use our for the frame update thing
    Object.keys(this.refs).forEach( (ref_name)=> this.refs[ref_name].forceUpdate() )
    //simpler: just one named 'frame': // this.refs.frame.forceUpdate()
  }
  checkbox_auto_update_clicked(){
    // if we're turning it on, might as well run it then
    if (this.state.auto_update == false) { this.run_code()}
    this.setState({auto_update: !this.state.auto_update})
  }
  moveCursor (line, ch){
    // setting it as a prop doesn't work after the initial render, so not for us
    this.refs.editor.cm.focus()
    this.refs.editor.cm.setCursor({line, ch})
  }
  render(){
    // don't render if we don't yet have the files, for now
    // will need to fix this for creating new gists
    if (!this.props.files){ return <div></div> }
    // console.log(this.props.files)
    // console.log(this.props.active_edit)
    let html_filenames = Object.keys(this.props.files).filter( (x)=>x.endsWith(".html"))
    let js_filenames = Object.keys(this.props.files).filter( (x)=>x.endsWith(".js"))
    let active_ext = this.props.active_edit.split('.')[this.props.active_edit.split('.').length-1]
    let editor_content = this.props.files[this.props.active_edit]["content"]
    let preview_frame_style = this.props.hide_preview_frame ? {display:'none'} : null
    let is_https = location.protocol=="http:"

    return <div className="lab">
      <div className="run_control">
        <a href="#" onClick={e=>{e.preventDefault();this.run_code()}}>Run</a>
        <label>
          <input type="checkbox" checked={this.state.auto_update} onChange={e=>this.checkbox_auto_update_clicked(e)} />
          <span>Auto Update</span>
        </label>
      </div>
      <FileBar files={this.props.files}
        active_edit={this.props.active_edit}
        switchFile={this.props.switchFile}
        addNewFile={this.props.addNewFile}
        whitelist={this.props.files_shown_white_list}/>
      <Editor ref="editor" value={editor_content} onBeforeChange={this.props.onBeforeChange } ext={active_ext} runPressed={this.run_code.bind(this)} theme={this.props.cm_theme}/>
      {html_filenames.map( (html_file, i)=>{
        let id = `${html_file}_${this.props.id}`.replace(/\./g, '')
        // if we wanted to do babel stuff, here's the place
        // MagicSandbox(template, files, supress_console_messages, is_https)
        let frame_content = MagicSandbox(this.props.files[html_file]["content"], this.props.files, true, is_https)
        let lines = this.state.output_lines.filter( x=> x.frame_id == id)
        let syntax_error = check_js_files_syntax(this.props.files)
        if (syntax_error){
          // redefining it
          lines = [{data: syntax_error.err.message, line:syntax_error.err.loc.line, column: syntax_error.err.loc.column, filename:syntax_error.filename, type:'syntax-error'}]
          // syntax_error =  {data: syntax_error.err.message, line:syntax_error.err.loc.line, column: syntax_error.err.loc.column, filename:syntax_error.filename}
          frame_content = false
        }
        return <div key={id} className="preview_group">
          {this.props.show_frame_filename ? <div className="frame_filename">{html_file}</div>:null}
          <Frame ref={`frame-${html_file.replace(/\./g, '')}`} id={id} content={frame_content} auto_update={this.state.auto_update} style={preview_frame_style}/>
          {this.props.hide_console_view ? null :<ConsoleView lines={lines} moveCursor={this.moveCursor}/>}

          </div>
      } )}
    </div>
  }
}


// todo, pass labprops to gist_lab , maybe lab data container
Lab.defaultProps = {
  files_shown_white_list: false, // [] for hiding it
  show_frame_filename: false,
  hide_preview_frame: false,
  hide_console_view: false,
//   // not impl yet:
//   show_save_as_new_button: true,
//   show_fork_button: true,
//   asset_string: '',
//   allow_anonymous: false,
//   babel_preset: '',
}
