import Hubkit from 'hubkit';
import LocalCache from './cache'
import LocalCredStore from './data_store_cred'
import {EventEmitter} from 'events'
// ? should I be using an ee specific for the browser
// https://www.npmjs.com/package/browser-event-emitter

// contains our gist client with caching to localstorage
// but it respects headers, 304 responses

// we export an instance of this, i.e. as a singleton
// ideally this could be used with gatekeeper or other server based auth

class GistStore {
  constructor(){
    this.ee = new EventEmitter()
    // this.gh =  new GitHub()
    this.gh = new Hubkit(this.default_options())
  }
  default_options(opt={}){
    // let default_opt = { allPages:false }
    // let result = Object.assign(default_opt, opt)
    opt['cache'] = LocalCache
    return opt
  }
  // we only want this to happen once automatically, but all the instances will
  // attempt to call it on mount
  maybe_login(){
    if (this.tried_cached_cred == true) { return }
    // console.log('maybe_login')
    this.tried_cached_cred = true
    let cred = LocalCredStore.get_cred()
    if (cred){ this.login(cred) }
  }
  login(opt){
    this.gh = new Hubkit(this.default_options(opt))
    // return when the login is done, for error handling
    // the emiter deals with updating most of the state
    return this.gh.request(`GET /users/:username`).then( data=>{
      this.ee.emit('login', {username:data.login, avatar_url:data.avatar_url})
      //
      this.get_all_user_gists().then( (all_gist_data) =>{
        this.ee.emit('user_gist_lookup_loaded', all_gist_data)
      })
    })
  }
  triggerLogOut(){
    this.gh = new Hubkit(this.default_options())
    this.ee.emit('logout')
  }
  get_gist_data(id){
    try {
      return this.gh.request(`GET /gists/${id}`).then( data=>{
          let files = extract_gist_data_files(data)
          let meta = extract_gist_meta(data)
          return Promise.resolve({files, meta})
        })
    } catch (e) {
      return new Promise( (resolve, reject)=> reject() )
    }
  }
  get_all_user_gists(){
    return this.gh.request(`GET /users/:username/gists`).then( data=>{
      let result = {}
      for (let gist of data){
        let desc = gist['description']
        if (result[desc]) { result[desc].push(gist['id']) }
        else {result[desc]= [gist['id']] }
      }
      return Promise.resolve(result)
    })

  }
  create_new_gist(files, description='Description', is_public=false){
    let body = {files, description, 'public': is_public }
    return this.gh.request(`POST /gists`, {body}).then( data=>{
        // let files = extract_gist_data_files(data)
        let meta = extract_gist_meta(data)
        return Promise.resolve(meta)
      })

  }
  // assumes you own this gist
  update_own_gist(id, files, description='Description'){
    console.log({description})
    let body = {files, description}
    return this.gh.request(`PATCH /gists/:id`, {id, body}).then( data=>{
        let meta = extract_gist_meta(data)
        return Promise.resolve(meta)
      })
  }
}

// Helpers

let unwanted_extensions = [".png", ".jpg", ".jpeg",".bmp", ".gif"]

// strip out all the data we don't care about
// from the github response, helper for get_gist_files
// and ignores image files
function extract_gist_data_files(data){
  let files = {}
  for (let [key, value] of Object.entries(data["files"]) ){
    // console.log(key)
    // console.log(value)
    if (!unwanted_extensions.find( x=>key.endsWith(x) ) ) {
      files[key] = {content:value.content}
    }

  }
  return files
}

// get our meta object
function extract_gist_meta(data){
  let description = data.description
  let owner = data.owner ? data.owner.login : false
  let updated_at = data.updated_at
  let retrieved_at = new Date()
  let meta = {owner, description, updated_at, retrieved_at, id: data.id, html_url:data.html_url}
  return meta
}

// XXX exporting an instance XXX
const GistDataStore = new GistStore()
window.gistds = GistDataStore
export default GistDataStore
