// Automatically load files data from code blocks on page, by closest heading
// and then insert a lab instance for each group of code blocks, removing the blocks.

// TODO,
// simpler version, just anything with data-gist-id= is now a lab.

import React from 'react';
import ReactDOM from 'react-dom'
import GistLab from '../components/gist_lab'
import UncontrolledLab from '../components/uncontrolled_lab'
import Lab from '../components/lab'
// import Lab from './components/lab'

function get_lab_data_from_page(root_el, c_sel, h_sel="h1,h2,h3,h4,h5,h6"){
  // first, go through headers and code blocks and give the code blocks
  // a data-title based on the last title
  let headers_and_code_blocks = root_el.querySelectorAll([h_sel, c_sel].join(',') )
  // let headers_and_code_blocks = document.querySelectorAll('h1,h2,h3,h4,h5,h6, pre code')
  let last_title = ''
  for (let el of headers_and_code_blocks){
    if (el.nodeName !='CODE'){
      last_title = el.textContent
    } else {
      el.dataset.title = last_title
    }
  }
  // now go through, using the titles from above to group them
  let labs = {}
  for (let code_block of  root_el.querySelectorAll(c_sel) ){
    let title = code_block.dataset.title
    // console.log(code_block.dataset)
    let filename = code_block.dataset.filename
    // some simple defaults, only works in the super simple case of
    // a lab consisting of one js and one html file
    if (!filename ){
      // assumes a single class XXX
      filename = code_block.classList.value.endsWith('-js') ? 'index.js' : ''
      filename = code_block.classList.value.endsWith('-html') ? 'index.html' : filename
    }
    if (!labs[title]){ labs[title]={} }
    labs[title][filename] = {content:code_block.textContent}
  }
  return labs
}

function replace_with_labs_from_page(root_el, labs, is_uncontrolled_lab, props, c_sel){
  let index = -1
  for (let [title, file] of Object.entries(labs)){
    index+=1
    let files = labs[title]
    // let pres = [].slice.call( document.querySelectorAll(`pre code[data-title="${title}"]`) )
    let pres = [].slice.call( root_el.querySelectorAll(`${c_sel}[data-title="${title}"]`) )
    let first = pres[0]
    let wrap = document.createElement('div')
    wrap.className = 'lab_wrap'
    first.parentNode.parentNode.replaceChild(wrap, first.parentNode)

    if (is_uncontrolled_lab){
      // todo this needs more methods, otherwise it'll be read only
      ReactDOM.render(<UncontrolledLab id={`lab-${index}`} initial_files={files} title={title} {...props} />, wrap)
      // ReactDOM.render(<UncontrolledLab id={`lab-${index}`} files={files} title={title} {...props} />, wrap)
    } else {
      ReactDOM.render(<GistLab id={`lab-${index}`} initial_files={files} title={title} {...props} />, wrap)
    }
    // remove all the pre's'n'codes
    pres.map( x=>x.parentNode.remove() )
  }
}

function replace_gists_with_labs(root_el, props, c_sel){
  // let gist_els = [].slice.call( root_el.querySelectorAll(c_sel) )
  for (let gist_el of root_el.querySelectorAll(c_sel) ){
    // index+=1
    // let files = labs[title]
    // let pres = [].slice.call( document.querySelectorAll(`pre code[data-title="${title}"]`) )
    // let pres = [].slice.call( root_el.querySelectorAll(`${c_sel}[data-title="${title}"]`) )
    // let first = pres[0]
    let wrap = document.createElement('div')
    wrap.className = 'lab_wrap'
    let gist_id = gist_el.dataset.gist_id
    let title = gist_el.dataset.title
    // console.log(gist_id)
    gist_el.parentNode.replaceChild(wrap, gist_el)

    ReactDOM.render(<GistLab gist_id={gist_id} title={title} {...props} />, wrap)

    gist_el.remove()
  }
}

// lab with gist wrapper
// pass ContainerLab props, including {lab_props}
export function GistLabsFromPage(props={}, root_el=document, c_selector="pre code"){
  let labs = get_lab_data_from_page(root_el, c_selector)
  replace_with_labs_from_page(root_el, labs, false, props, c_selector)
}
// no github gist integration, and currently not editable because files is a prop of Lab
//
// pass lab_props
export function UncontrolledLabsFromPage(props={}, root_el=document, c_selector="pre code"){
  let labs = get_lab_data_from_page(root_el, c_selector)
  replace_with_labs_from_page(root_el, labs, true, props, c_selector)
}

export function LabsFromGists(props={}, root_el=document, c_selector="div[data-gist_id]"){
  replace_gists_with_labs(root_el, props, c_selector)
}
